@@ -5,7 +5,7 @@ import Data.List  as List  (find, transpose, unfoldr, isPrefixOf,
 import Data.Maybe as Maybe (fromMaybe, catMaybes, isJust, mapMaybe, )
 import Data.Maybe.HT       (toMaybe, )
 import Control.Monad.HT    ((<=<), )
-import Control.Monad       (guard, msum, mplus, )
+import Control.Monad       (guard, msum, mplus, liftM2, )
 import Control.Applicative ((<$>), (<*>), )
 import Data.Tuple.HT       (mapPair, mapFst, mapSnd, forcePair, swap, )
 
@@ -952,11 +952,11 @@ foldl'r, foldl'rStrict, foldl'rNaive ::
    (b -> a -> b) -> b -> (c -> d -> d) -> d -> [(a,c)] -> (b,d)
 foldl'r f b0 g d0 =
 --   (\(k,d1) -> (k b0, d1)) .
-   mapFst ($b0) .
+   mapFst ($ b0) .
    foldr (\(a,c) ~(k,d) -> (\b -> k $! f b a, g c d)) (id,d0)
 
 foldl'rStrict f b0 g d0 =
-   mapFst ($b0) .
+   mapFst ($ b0) .
    foldr (\(a,c) ~(k,d) -> ((,) $! (\b -> k $! f b a)) $! g c d) (id,d0)
 
 foldl'rNaive f b g d xs =
@@ -1074,7 +1074,7 @@ rotate n x =
 
 {- | more efficient implementation of rotate'
 
-prop> \(NonNegative n) (NonEmpty xs) -> rotate n xs == rotate' n (xs::String)
+prop> \n (NonEmpty xs) -> rotate n xs == rotate' n (xs::String)
 -}
 rotate' n x =
    uncurry (flip (++))
@@ -1169,6 +1169,47 @@ mapAdjacent1 f a xs =
    zipWith (\a0 (a1,b) -> f a0 a1 b) (a : map fst xs) xs
 
 
+{- |
+>>> equalWith (<=) "ab" "bb"
+True
+>>> equalWith (<=) "aa" "bbb"
+False
+>>> equalWith (==) "aa" "aaa"
+False
+
+prop> \as bs -> let f a b = abs (a-b) <= (10::Int) in equalWith f as bs ==  equalWithRec f as bs
+prop> \as bs -> let f a b = abs (a-b) <= (10::Int) in equalWith f as bs ==  equalWithLiftM f as bs
+-}
+equalWith, equalWithLiftM, equalWithRec ::
+   (a -> b -> Bool) -> [a] -> [b] -> Bool
+equalWith f as bs =
+   and $
+   zipWith
+      (\ma mb ->
+         case (ma,mb) of
+            (Just a, Just b) -> f a b
+            (Nothing, Nothing) -> True
+            _ -> False)
+      (map Just as ++ [Nothing])
+      (map Just bs ++ [Nothing])
+
+equalWithLiftM f as bs =
+   all (Just True ==) $
+   zipWith
+      (\ma mb ->
+         case (ma,mb) of
+            (Nothing, Nothing) -> Just True
+            _ -> liftM2 f ma mb)
+      (map Just as ++ [Nothing])
+      (map Just bs ++ [Nothing])
+
+equalWithRec f =
+   let go (a:as) (b:bs) = f a b && go as bs
+       go [] [] = True
+       go _ _ = False
+   in go
+
+
 {- |
 Enumerate without Enum context.
 For Enum equivalent to enumFrom.
